/*
 * printk driver
 */

/* Kernel includes */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#if  LINUX_VERSION_CODE >= 0x20100
#include <asm/uaccess.h>
#else
#include <asm/segment.h>
#endif

#if  LINUX_VERSION_CODE >= 0x20100
static long pk_write(struct inode *node, struct file * file,const char * buf,unsigned long count)
#else
static int pk_write(struct inode *node, struct file * file,const char * buf,int count)
#endif
{
	int left, c;

	for (left = count; left > 0; left--) {
#if  LINUX_VERSION_CODE >= 0x20100
		get_user(c,buf++);
#else
		c = get_fs_byte(buf++);
#endif
		printk("%c",c);
	}
	return count;
}

/*
 * Our special open code.
 * MOD_INC_USE_COUNT make sure that the driver memory is not freed
 * while the device is in use.
 */
static int
pk_open( struct inode* ino, struct file* filep)
{
   MOD_INC_USE_COUNT;
   return 0;   
}

/*
 * Now decrement the use count.
 */
static void
pk_close( struct inode* ino, struct file* filep)
{
   MOD_DEC_USE_COUNT;
}

static struct file_operations pk_fops = {
	NULL,
	NULL,
	pk_write,
	NULL,		/* pk_readdir */
	NULL,		/* pk_select */
	NULL,		/* pk_ioctl */
	NULL,
	pk_open,
	pk_close,
	NULL		/* fsync */
};


/*
 * And now the modules code and kernel interface.
 */

static int PK_MAJOR=31;

#ifdef __cplusplus
extern "C" {
#endif

int _42;

int
init_module( void) {
	printk( "printk:  init_module called\n");
	if (register_chrdev(PK_MAJOR, "user-printk", &pk_fops)) {
	  return -EIO;
	}

	return 0;
}

void
cleanup_module( void) {
  printk( "printk:  cleanup_module\n");

  if (unregister_chrdev(PK_MAJOR, "user-printk") != 0) {
    printk("cleanup_module failed\n");
  }
}



#ifdef __cplusplus
}
#endif

